package consume

import (
	"encoding/json"
	"query-service/internal/adapters/brokeradapter"
	"query-service/pkg/errorhandler"
	"strings"
)

// HandleMessage gets called when a message is received
func (s *Service) HandleMessage(msg *brokeradapter.Message) {
	// Grab sessionID and clientID from the headers
	sessionID, ok := msg.Headers["sessionID"].(string)
	if !ok {
		return
	}

	clientID, ok := msg.Headers["clientID"].(string)
	if !ok {
		return
	}

	// Convert the json byte msg to a query string
	query, databaseName, err := s.queryConverter.ConvertQuery(&msg.Body)
	if err != nil {
		errorMsg := make(map[string]string)
		errorMsg["type"] = "query_translation_error"
		errorMsg["value"] = err.Error()
		errorMsgBytes, _ := json.Marshal(errorMsg)
		s.producer.PublishMessage(&errorMsgBytes, &sessionID)
		return
	}

	// Check if there was a database name
	if *databaseName == "" {
		errorMsg := make(map[string]string)
		errorMsg["type"] = "query_malformed_request_error"
		errorMsg["value"] = "no database name supplied"
		errorMsgBytes, _ := json.Marshal(errorMsg)
		s.producer.PublishMessage(&errorMsgBytes, &sessionID)
		return
	}

	// Remove newlines and tabs from string
	queryNoNewlines := strings.ReplaceAll(*query, "\n", "")
	queryNoTabs := strings.ReplaceAll(queryNoNewlines, "\t", "")

	// Send the resulting query back to the client
	msgmap := make(map[string]interface{})
	msgmap["type"] = "query_translation_result"
	msgmap["values"] = queryNoTabs
	msgbyte, err := json.Marshal(msgmap)
	if err != nil {
		errorhandler.LogError(err, "Marshalling query_translation_result went wrong!") // TODO: send error message to client instead
		return
	}
	s.producer.PublishMessage(&msgbyte, &sessionID) // TODO: should this be a go routine?

	// Fetch database credentials from the user service
	databaseInfo, err := s.databaseInfoService.GetDatabaseInfo(&clientID, databaseName)

	// execute and retrieve result
	// convert result to general (node-link (?)) format
	result, err := s.requestSender.SendAQLQuery(*query, databaseInfo.Username, databaseInfo.Password, databaseInfo.URL, databaseInfo.Port, databaseInfo.InternalDatabaseName)
	if err != nil {
		errorMsg := make(map[string]string)
		errorMsg["type"] = "query_database_error"
		errorMsg["value"] = err.Error()
		errorMsgBytes, _ := json.Marshal(errorMsg)
		s.producer.PublishMessage(&errorMsgBytes, &sessionID)
		return
	}

	// Add type indicator to result from database
	resultMsgMap := make(map[string]interface{})
	resultMsgMap["type"] = "query_result"
	resultMsgMap["values"] = *result
	resultMsgByte, err := json.Marshal(resultMsgMap)
	if err != nil {
		errorhandler.LogError(err, "Marshalling query_result went wrong!") // TODO: send error message to client instead
		return
	}

	s.producer.PublishMessage(&resultMsgByte, &sessionID)
}
